@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 1998 - 2016
+  Copyright (c) 1998 - 2018
   CLST  - Radboud University
   ILK   - Tilburg University
   CLiPS - University of Antwerp
@@ -94,10 +94,9 @@ void startExperiments( ServerBase *server ){
     }
   }
   if ( allvals.empty() ){
-    cerr << "Unable to initalize at least one tagger" << endl
-	 << "please check your commandline and/or your confiuration file"
-	 << endl;
-    exit(EXIT_FAILURE);
+    string mess = "TimblServer: Unable to initalize at least one experiment\n";
+    mess += "please check your commandline or configuration file";
+    throw runtime_error( mess );
   }
 
   auto experiments = new map<string, TimblExperiment*>();
@@ -105,6 +104,7 @@ void startExperiments( ServerBase *server ){
 
   map<string,string>::iterator it = allvals.begin();
   while ( it != allvals.end() ){
+    string exp_name = it->first;
     TiCC::CL_Options opts;
     opts.set_short_options( timbl_short_opts + serv_short_opts );
     opts.set_long_options( timbl_long_opts + serv_long_opts );
@@ -120,16 +120,16 @@ void startExperiments( ServerBase *server ){
     if ( opts.is_present( 'a', value ) ){
       // the user gave an algorithm
       if ( !string_to( value, algorithm ) ){
-	cerr << "illegal -a value: " << value << endl;
-	exit(EXIT_FAILURE);
+	string mess = exp_name + ":start(): illegal -a value: " + value;
+	throw runtime_error( mess );
       }
     }
     if ( !opts.extract( 'f', trainName ) )
       opts.extract( 'i', treeName );
     if ( opts.extract( 'u', ProbInFile ) ){
       if ( algorithm == IGTREE ){
-	cerr << "-u option is useless for IGtree" << endl;
-	exit(EXIT_FAILURE);
+	string mess = exp_name + ":start(): -u option is useless for IGtree";
+	throw runtime_error( mess );
       }
     }
     if ( opts.extract( 'w', value ) ){
@@ -140,8 +140,9 @@ void startExperiments( ServerBase *server ){
 	size_t num = TiCC::split_at( value, parts, ":" );
 	if ( num == 2 ){
 	  if ( !string_to( parts[1], W ) ){
-	    cerr << "invalid weighting option: " << value << endl;
-	    exit(1);
+	    string mess = exp_name + ":start(): invalid weighting option:"
+	      + value;
+	    throw runtime_error( mess );
 	  }
 	  WgtInFile = parts[0];
 	  WgtType = W;
@@ -150,14 +151,15 @@ void startExperiments( ServerBase *server ){
 	  WgtInFile = value;
 	}
 	else {
-	  cerr << "invalid weighting option: " << value << endl;
-	  exit(1);
+	  string mess = exp_name + ":start(): invalid weighting option:"
+	    + value;
+	  throw runtime_error( mess );
 	}
       }
     }
     opts.extract( "matrixin", MatrixInFile );
     if ( !( treeName.empty() && trainName.empty() ) ){
-      TimblAPI *run = new TimblAPI( opts, it->first );
+      TimblAPI *run = new TimblAPI( opts, exp_name );
       bool result = false;
       if ( run && run->Valid() ){
 	if ( treeName.empty() ){
@@ -179,19 +181,19 @@ void startExperiments( ServerBase *server ){
       }
       if ( result ){
 	run->initExperiment();
-	(*experiments)[it->first] = run->grabAndDisconnectExp();
+	(*experiments)[exp_name] = run->grabAndDisconnectExp();
 	delete run;
-	server->myLog << "started experiment " << it->first
+	server->myLog << "started experiment " << exp_name
 		      << " with parameters: " << it->second << endl;
       }
       else {
-	server->myLog << "FAILED to start experiment " << it->first
+	server->myLog << "FAILED to start experiment " << exp_name
 		      << " with parameters: " << it->second << endl;
       }
     }
     else {
       server->myLog << "missing '-i' or '-f' option in serverconfig entry: '"
-		    << it->first << "=" << it->second << "'" << endl;
+		    << exp_name << "=" << it->second << "'" << endl;
     }
     ++it;
   }
@@ -200,13 +202,13 @@ void startExperiments( ServerBase *server ){
 class TcpServer : public TcpServerBase {
 public:
   void callback( childArgs* );
-  TcpServer( const TiCC::Configuration *c ): TcpServerBase( c ){};
+  explicit TcpServer( const TiCC::Configuration *c ): TcpServerBase( c ){};
 };
 
 class HttpServer : public HttpServerBase {
 public:
   void callback( childArgs* );
-  HttpServer( const TiCC::Configuration *c ): HttpServerBase( c ){};
+  explicit HttpServer( const TiCC::Configuration *c ): HttpServerBase( c ){};
 };
 
 TimblExperiment *createClient( const TimblExperiment *exp,
@@ -224,34 +226,34 @@ TimblExperiment *createClient( const TimblExperiment *exp,
   return result;
 }
 
-  enum CommandType { UnknownCommand, Classify, Base,
-		     Query, Set, Exit, Comment };
+enum CommandType { UnknownCommand, Classify, Base,
+		   Query, Set, Exit, Comment };
 
-  CommandType check_command( const string& com ){
-    CommandType result = UnknownCommand;
-    if ( compare_nocase_n( com, "CLASSIFY" ) )
-      result = Classify;
-    else if ( compare_nocase_n( com, "QUERY" ) )
-      result = Query;
-    else if ( compare_nocase_n( com, "BASE") )
-      result = Base;
-    else if ( compare_nocase_n( com, "SET") )
-      result = Set;
-    else if ( compare_nocase_n( com, "EXIT" ) )
-      result = Exit;
-    else if ( com[0] == '#' )
-      result = Comment;
-    return result;
-  }
+CommandType check_command( const string& com ){
+  CommandType result = UnknownCommand;
+  if ( compare_nocase_n( com, "CLASSIFY" ) )
+    result = Classify;
+  else if ( compare_nocase_n( com, "QUERY" ) )
+    result = Query;
+  else if ( compare_nocase_n( com, "BASE") )
+    result = Base;
+  else if ( compare_nocase_n( com, "SET") )
+    result = Set;
+  else if ( compare_nocase_n( com, "EXIT" ) )
+    result = Exit;
+  else if ( com[0] == '#' )
+    result = Comment;
+  return result;
+}
 
 inline void Split( const string& line, string& com, string& rest ){
-  string::size_type pos = line.find_first_of(" \t");
-  if ( pos != string::npos ){
-    com = line.substr(0,pos);
-    rest = line.substr(pos+1);
+  vector<string> parts = TiCC::split( line, 2 );
+  if ( parts.size() == 2 ){
+    com = parts[0];
+    rest = parts[1];
   }
   else {
-    com = line;
+    com = parts[0];
     rest = "";
   }
 }
@@ -357,13 +359,11 @@ void TcpServer::callback( childArgs *args ){
     static_cast<map<string, TimblExperiment*> *>(callback_data);
 
   int result = 0;
-  string baseName;
   args->os() << "Welcome to the Timbl server." << endl;
   if ( experiments->size() == 1
        && experiments->find("default") != experiments->end() ){
-    baseName = "default";
     DBG << " Voor Create Default Client " << endl;
-    client = new TimblClient( (*experiments)[baseName], args );
+    client = new TimblClient( (*experiments)["default"], args );
     DBG << " Na Create Client " << endl;
     // report connection to the server terminal
     //
@@ -394,7 +394,6 @@ void TcpServer::callback( childArgs *args ){
 	map<string,TimblExperiment*>::const_iterator it
 	  = experiments->find(Param);
 	if ( it != experiments->end() ){
-	  baseName = Param;
 	  args->os() << "selected base: '" << Param << "'" << endl;
 	  if ( client )
 	    delete client;
@@ -404,7 +403,7 @@ void TcpServer::callback( childArgs *args ){
 	  // report connection to the server terminal
 	  //
 	  char line[256];
-	  sprintf( line, "Thread %zd, on Socket %d",
+	  sprintf( line, "Thread %lu, on Socket %d",
 		   (uintptr_t)pthread_self(), sockId );
 	  LOG << line << ", started." << endl;
 	}
@@ -468,33 +467,31 @@ void TcpServer::callback( childArgs *args ){
             (((x) >= 'A') && ((x) <= 'F')))
 
 
-  string urlDecode( const string& s ) {
-    int cc;
-    string result;
-    int len=s.size();
-    for (int i=0; i<len ; ++i ) {
-      cc=s[i];
-      if (cc == '+') {
-	result += ' ';
-      }
-      else if ( cc == '%' &&
-		( i < len-2 &&
-		  ( IS_HEX(s[i+1]) ) &&
-		  ( IS_HEX(s[i+2]) ) ) ){
-	std::istringstream ss( "0x"+s.substr(i+1,2) );
-	int tmp;
-	ss >> std::showbase >> std::hex;
-	ss >> tmp;
+string urlDecode( const string& s ) {
+  string result;
+  size_t len=s.size();
+  for ( size_t i=0; i<len ; ++i ) {
+    int cc=s[i];
+    if (cc == '+') {
+      result += ' ';
+    }
+    else if ( cc == '%' &&
+	      ( i < len-2 &&
+		( IS_HEX(s[i+1]) ) &&
+		( IS_HEX(s[i+2]) ) ) ){
+      std::istringstream ss( "0x"+s.substr(i+1,2) );
+      int tmp;
+      ss >> std::showbase >> std::hex;
+      ss >> tmp;
       result = result + (char)tmp;
       i += 2;
-      }
-      else {
-	result += cc;
-      }
     }
-    return result;
+    else {
+      result += cc;
+    }
   }
-
+  return result;
+}
 
 void HttpServer::callback( childArgs *args ){
   // process the test material
@@ -504,7 +501,7 @@ void HttpServer::callback( childArgs *args ){
   map<string, TimblExperiment*> *experiments =
     static_cast<map<string, TimblExperiment*> *>(callback_data);
   char logLine[256];
-  sprintf( logLine, "Thread %zd, on Socket %d", (uintptr_t)pthread_self(),
+  sprintf( logLine, "Thread %lu, on Socket %d", (uintptr_t)pthread_self(),
 	   args->id() );
   LOG << logLine << ", started." << endl;
   string Line;
@@ -544,25 +541,17 @@ void HttpServer::callback( childArgs *args ){
 		TiCC::XmlDoc doc( "TiMblResult" );
 		xmlNode *root = doc.getRoot();
 		TiCC::XmlSetAttribute( root, "algorithm", TiCC::toString(api->Algorithm()) );
-		vector<string> avs;
-		int avNum = TiCC::split_at( qstring, avs, "&" );
-		if ( avNum > 0 ){
+		vector<string> avs = TiCC::split_at( qstring, "&" );
+		if ( !avs.empty() ){
 		  multimap<string,string> acts;
-		  for ( int i=0; i < avNum; ++i ){
-		    vector<string> parts;
-		    int num = TiCC::split_at( avs[i], parts, "=" );
-		    if ( num == 2 ){
+		  for ( const auto& av : avs ){
+		    vector<string> parts = TiCC::split_at( av, "=", 2 );
+		    if ( parts.size() == 2 ){
 		      acts.insert( make_pair(parts[0], parts[1]) );
 		    }
-		    else if ( num > 2 ){
-		      string tmp = parts[1];
-		      for( int i=2; i < num; ++i )
-			tmp += string("=")+parts[i];
-		      acts.insert( make_pair(parts[0], tmp ) );
-		    }
 		    else {
 		      LS << "unknown word in query "
-			 << avs[i] << endl;
+			 << av << endl;
 		    }
 		  }
 		  typedef multimap<string,string>::const_iterator mmit;
@@ -688,7 +677,7 @@ int main(int argc, char *argv[]){
     // Start.
     //
     cerr << "TiMBL Server " << TimblServer::Version()
-	 << " (c) CLTS/ILK/CLIPS 1998 - 2016.\n"
+	 << " (c) CLTS/ILK/CLIPS 1998 - 2018.\n"
 	 << "Tilburg Memory Based Learner\n"
 	 << "Centre for Language and Speech Technology, Radboud University\n"
 	 << "Induction of Linguistic Knowledge Research Group, Tilburg University\n"
@@ -704,7 +693,6 @@ int main(int argc, char *argv[]){
     opts.set_short_options( timbl_short_opts + serv_short_opts );
     opts.set_long_options( timbl_long_opts + serv_long_opts );
     opts.init( argc, argv );
-    string value;
     if ( opts.is_present( 'h' )
 	 || opts.is_present( "help" ) ){
       usage_full();
@@ -743,16 +731,12 @@ int main(int argc, char *argv[]){
     cerr << "ran out of memory somewhere" << endl;
     cerr << "timblserver terminated, Sorry for that" << endl;
   }
-  catch(std::string& what){
-    cerr << "an exception was raised: '" << what << "'" << endl;
-    cerr << "timblserver terminated, Sorry for that" << endl;
-  }
   catch(std::exception& e){
     cerr << "a standard exception was raised: '" << e.what() << "'" << endl;
     cerr << "timblserver terminated, Sorry for that" << endl;
   }
   catch(...){
-    cerr << "some exception was raised" << endl;
+    cerr << "some unhandled exception was raised" << endl;
     cerr << "timblserver terminated, Sorry for that" << endl;
   }
   return 0;
